@@ -37,7 +37,7 @@ static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/c
 #include <ctype.h>
 #include <sys/types.h>
 #include <arpa/inet.h>	/* for htons() */
-#ifdef	SYSV
+#if defined(SYSV) || defined(EMSCRIPTEN)
 #include <time.h>
 #else
 #include <sys/time.h>
@@ -60,6 +60,7 @@ static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/c
 #include "utils/styles.h"
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
+#include "extract/extractInt.h"	/* for LabelList */
 #include "utils/main.h"		/* for Path and CellLibPath */
 #include "utils/stack.h"
 
@@ -122,7 +123,7 @@ typedef struct {
 #define GDS_UNPROCESSED CLIENTDEFAULT
 #define GDS_PROCESSED   1
 
-#define PUSHTILE(tp) \
+#define PUSHTILEZ(tp) \
     if ((tp)->ti_client == (ClientData) GDS_UNPROCESSED) { \
         (tp)->ti_client = (ClientData) GDS_PENDING; \
         STACKPUSH((ClientData) (tp), SegStack); \
@@ -824,7 +825,8 @@ calmaProcessDefZ(def, outf, do_library)
 {
     char *filename;
     int polyidx;
-    bool isReadOnly, oldStyle, hasContent, isAbstract, hasGDSEnd;
+    bool isReadOnly, oldStyle, hasContent, isAbstract, hasGDSEnd, needHier;
+    bool hierWrite, arrayWrite;
     HashEntry *he;
 
     /* Skip if already output */
@@ -840,11 +842,8 @@ calmaProcessDefZ(def, outf, do_library)
 
     /* Read the cell in if it is not already available. */
     if ((def->cd_flags & CDAVAILABLE) == 0)
-    {
-	bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
-	if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL))
+	if (!DBCellRead(def, TRUE, TRUE, NULL))
 	    return (0);
-    }
 
     /*
      * Flag an error if attempting to write the default (UNNAMED) cell
@@ -873,6 +872,7 @@ calmaProcessDefZ(def, outf, do_library)
     DBPropGet(def, "LEFview", &isAbstract);
     DBPropGet(def, "GDS_START", &hasContent);
     DBPropGet(def, "GDS_END", &hasGDSEnd);
+    DBPropGet(def, "CIFhier", &needHier);
     filename = (char *)DBPropGet(def, "GDS_FILE", &isReadOnly);
 
     /* When used with "calma addendum true", don't output the read-only	*/
@@ -902,8 +902,29 @@ calmaProcessDefZ(def, outf, do_library)
      * as they are expected to be in the referenced GDS file.
      */
     if (!hasContent || hasGDSEnd)
-	if (DBCellEnum(def, calmaProcessUseZ, (ClientData) outf) != 0)
+    {
+	int result;
+
+	if (needHier)
+	{
+	    hierWrite = CIFHierWriteDisable;
+	    arrayWrite = CIFArrayWriteDisable;
+
+	    CIFHierWriteDisable = FALSE;
+	    CIFArrayWriteDisable = FALSE;
+	}
+
+	result = DBCellEnum(def, calmaProcessUseZ, (ClientData) outf);
+
+	if (needHier)
+	{
+	    CIFHierWriteDisable = hierWrite;
+	    CIFArrayWriteDisable = arrayWrite;
+	}
+
+	if (result != 0)
 	    return 1;
+    }
 
     /* Give some feedback to the user */
     TxPrintf("   Generating output for cell %s\n", def->cd_name);
@@ -1106,8 +1127,25 @@ calmaProcessDefZ(def, outf, do_library)
     /* Output this cell definition from the Magic database */
     if (!isReadOnly)
 	if (!do_library)
+	{
+	    if (needHier)
+	    {
+		hierWrite = CIFHierWriteDisable;
+		arrayWrite = CIFArrayWriteDisable;
+	
+		CIFHierWriteDisable = FALSE;
+		CIFArrayWriteDisable = FALSE;
+	    }
+
 	    calmaOutFuncZ(def, outf, &TiPlaneRect);
 
+	    if (needHier)
+	    {
+		CIFHierWriteDisable = hierWrite;
+		CIFArrayWriteDisable = arrayWrite;
+	    }
+	}
+
     return 0;
 }
 
@@ -1146,6 +1184,7 @@ calmaOutFuncZ(def, f, cliprect)
     calmaOutputStructZ cos;
     bool propfound;
     char *propvalue;
+    extern int compport();	/* Forward declaration */
 
     cos.f = f;
     cos.area = (cliprect == &TiPlaneRect) ? NULL : cliprect;
@@ -1218,10 +1257,11 @@ calmaOutFuncZ(def, f, cliprect)
     CIFErrorDef = def;
     CIFGen(def, def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, FALSE,
 		(ClientData)f);
+
     if (!CIFHierWriteDisable)
-	CIFGenSubcells(def, &bigArea, CIFPlanes);
+        CIFGenSubcells(def, &bigArea, CIFPlanes);
     if (!CIFArrayWriteDisable)
-	CIFGenArrays(def, &bigArea, CIFPlanes);
+        CIFGenArrays(def, &bigArea, CIFPlanes);
 
     for (type = 0; type < CIFCurStyle->cs_nLayers; type++)
     {
@@ -1249,7 +1289,8 @@ calmaOutFuncZ(def, f, cliprect)
 
     if (CalmaDoLabels)
     {
-	int i, ltype, maxport = -1;
+	int i, ltype, numports = 0;
+	LabelList *ll = NULL, *newll;
 
 	for (lab = def->cd_labels; lab; lab = lab->lab_next)
 	{
@@ -1261,23 +1302,42 @@ calmaOutFuncZ(def, f, cliprect)
 	    }
 	    else
 	    {
-		if ((int)lab->lab_port > maxport)
-		    maxport = (int)lab->lab_port;
+		newll = (LabelList *)mallocMagic(sizeof(LabelList));
+		newll->ll_label = lab;
+		newll->ll_attr = (unsigned int)lab->lab_port;
+		newll->ll_next = ll;
+		ll = newll;
+		numports++;
 	    }
 	}
-	if (maxport >= 0)
-	    for (i = 0; i <= maxport; i++)
-		for (lab = def->cd_labels; lab; lab = lab->lab_next)
-		{
-		    ltype = CIFCurStyle->cs_portText[lab->lab_type];
-		    type = CIFCurStyle->cs_portLayer[lab->lab_type];
-		    if ((type >= 0) && ((lab->lab_flags & PORT_DIR_MASK) != 0) &&
-				(lab->lab_port == i))
-		    {
-			calmaWriteLabelFuncZ(lab, ltype, type, f);
-			/* break; */  /* Do not limit to unique labels! */
-		    }
-		}
+	if (newll != NULL)
+	{
+	    /* Turn linked list into an array, then run qsort on it	*/
+	    /* to sort by port number.					*/
+
+	    PortLabel *pllist = (PortLabel *)mallocMagic(numports * sizeof(PortLabel));
+	    i = 0;
+	    while (ll != NULL)
+	    {
+		pllist[i].pl_label = ll->ll_label;
+		pllist[i].pl_port = (unsigned int)ll->ll_attr;
+		freeMagic(ll);
+		ll = ll->ll_next;
+		i++;
+	    }
+
+	    qsort(pllist, numports, sizeof(PortLabel), compport);
+
+	    for (i = 0; i < numports; i++)
+	    {
+		lab = pllist[i].pl_label;
+		ltype = CIFCurStyle->cs_portText[lab->lab_type];
+		type = CIFCurStyle->cs_portLayer[lab->lab_type];
+		if (type >= 0)
+		    calmaWriteLabelFuncZ(lab, ltype, type, f);
+	    }
+	    freeMagic(pllist);
+	}
     }
 
     /* End of structure */
@@ -1696,7 +1756,7 @@ calmaWriteContactsZ(f)
             /* Get clip bounds, so that residue surround is	*/
 	    /* minimum.  Note that these values are in CIF/GDS	*/
 	    /* units, and the clipping rectangle passed to	*/
-	    /* calmaOutFunc is also in CIF/GDS units.		*/
+	    /* calmaOutFuncZ is also in CIF/GDS units.		*/
 
 	    halfsize = CIFGetContactSize(type, NULL, NULL, NULL) >> 1;
 
@@ -1841,7 +1901,7 @@ calmaMergePaintFuncZ(tile, cos)
     if (SegStack == (Stack *)NULL)
 	SegStack = StackNew(64);
 
-    PUSHTILE(tile);
+    PUSHTILEZ(tile);
     while (!StackEmpty(SegStack))
     {
 	t = (Tile *) STACKPOP(SegStack);
@@ -2009,7 +2069,7 @@ calmaMergePaintFuncZ(tile, cos)
 	    intedges += calmaAddSegment(&lb, is_ext,
 			MIN(RIGHT(t), RIGHT(tp)), TOP(t),
 			MAX(LEFT(t), LEFT(tp)), TOP(t));
-	    if (!is_ext) PUSHTILE(tp);
+	    if (!is_ext) PUSHTILEZ(tp);
 	}
 
 	if (split_type == 0x3)
@@ -2029,7 +2089,7 @@ left_search:
 	    intedges += calmaAddSegment(&lb, is_ext,
 			LEFT(t), MIN(TOP(t), TOP(tp)),
 			LEFT(t), MAX(BOTTOM(t), BOTTOM(tp)));
-	    if (!is_ext) PUSHTILE(tp);
+	    if (!is_ext) PUSHTILEZ(tp);
 	}
 
 	if (split_type == 0x0)
@@ -2049,7 +2109,7 @@ bottom_search:
 	    intedges += calmaAddSegment(&lb, is_ext,
 			MAX(LEFT(t), LEFT(tp)), BOTTOM(t),
 			MIN(RIGHT(t), RIGHT(tp)), BOTTOM(t));
-	    if (!is_ext) PUSHTILE(tp);
+	    if (!is_ext) PUSHTILEZ(tp);
 	}
 
 	if (split_type == 0x1)
@@ -2068,7 +2128,7 @@ right_search:
 	    intedges += calmaAddSegment(&lb, is_ext,
 			RIGHT(t), MAX(BOTTOM(t), BOTTOM(tp)),
 			RIGHT(t), MIN(TOP(t), TOP(tp)));
-	    if (!is_ext) PUSHTILE(tp);
+	    if (!is_ext) PUSHTILEZ(tp);
 	}
 
 	/* If tile is isolated, process it now and we're done */
