@@ -305,6 +305,7 @@ cifOut(outf)
     FILE *outf;
 {
     CellDef *def;
+    bool needHier;
 
     while (!StackEmpty(cifStack))
     {
@@ -316,10 +317,8 @@ cifOut(outf)
 
 	/* Read the cell in if it is not already available. */
 	if ((def->cd_flags & CDAVAILABLE) == 0)
-	{
-	    bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
-	    if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL)) continue;
-	}
+	    if (!DBCellRead(def, TRUE, TRUE, NULL))
+		continue;
 
 	/* Add any subcells to the stack.  This must be done before
 	 * outputting CIF to make sure that the subcells all have
@@ -385,10 +384,12 @@ cifOutFunc(def, f)
     CIFErrorDef = def;
     CIFGen(def, def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, FALSE,
 		(ClientData)NULL);
+
     if (!CIFHierWriteDisable)
-	CIFGenSubcells(def, &bigArea, CIFPlanes);
+        CIFGenSubcells(def, &bigArea, CIFPlanes);
     if (!CIFArrayWriteDisable)
-	CIFGenArrays(def, &bigArea, CIFPlanes);
+        CIFGenArrays(def, &bigArea, CIFPlanes);
+
     for (type = 0; type < CIFCurStyle->cs_nLayers; type++)
     {
 	layer = CIFCurStyle->cs_layers[type];
